package cc.fascinated.bat.command;

import lombok.Getter;
import lombok.NonNull;
import lombok.Setter;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import net.dv8tion.jda.api.interactions.commands.DefaultMemberPermissions;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import net.dv8tion.jda.api.interactions.commands.build.SlashCommandData;
import net.dv8tion.jda.api.interactions.commands.build.SubcommandGroupData;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author Braydon
 */
@Setter@Getter
public abstract class BatCommand extends ListenerAdapter implements IBatCommand {
    /**
     * The snowflake of this command, if it
     * was successfully registered on Discord.
     */
    private Long snowflake;

    /**
     * The data for this command.
     */
    @NonNull private final SlashCommandData commandData;

    /**
     * The category of this command.
     */
    @NonNull private final CommandCategory category;

    /**
     * The sub-commands for this command.
     */
    @NonNull private final List<BatSubCommand> subCommands = Collections.synchronizedList(new ArrayList<>());

    public BatCommand(@NonNull String name, @NonNull String description, @NonNull CommandCategory category, @NonNull OptionData... options) {
        this(name, description, category, DefaultMemberPermissions.ENABLED, options);
    }

    public BatCommand(@NonNull String name, @NonNull String description, @NonNull CommandCategory category, @NonNull DefaultMemberPermissions defaultPermissions, @NonNull OptionData... options) {
        commandData = Commands.slash(name, description)
                .setDefaultPermissions(defaultPermissions)
                .addOptions(options);
        this.category = category;
    }

    /**
     * Get the name of this command.
     *
     * @return the command name
     */
    @Override @NonNull
    public String getName() {
        return commandData.getName();
    }

    /**
     * Get the full name of this command.
     *
     * @return the full name
     */
    @Override @NonNull
    public String getFullName() {
        return getName();
    }

    /**
     * Get the description of this command.
     *
     * @return the command description
     */
    @Override @NonNull
    public String getDescription() {
        return commandData.getDescription();
    }

    /**
     * Get the mentionable name of this command.
     *
     * @return the mentionable name
     */
    @Override @NonNull
    public final String getMentionable() {
        return "</%s:%s>".formatted(getName(), getSnowflake());
    }

    /**
     * Register a sub-command to this command.
     *
     * @param subCommand the sub-command to register
     */
    protected final void withSubCommand(@NonNull BatSubCommand subCommand) {
        subCommands.add(subCommand);

        // Add the sub-command to the command data
        if (subCommand.getGroupName() == null) { // No command group
            getCommandData().addSubcommands(subCommand.getCommandData());
        } else {
            // Add to any existing command group, otherwise create a new one
            for (SubcommandGroupData subCommandGroup : getCommandData().getSubcommandGroups()) {
                if (subCommand.getGroupName().equals(subCommandGroup.getName())) {
                    subCommandGroup.addSubcommands(subCommand.getCommandData());
                    return;
                }
            }
            getCommandData().addSubcommandGroups(new SubcommandGroupData(
                    subCommand.getGroupName(),
                    "The " + subCommand.getGroupName() + " group for /" + getName() + " " + subCommand.getName()
            ).addSubcommands(subCommand.getCommandData()));
        }
    }

    /**
     * Get a sub-command by name.
     *
     * @param group the sub-command group, null if none
     * @param name  the sub-command name
     * @return the sub-command, null if not found
     */
    public final BatSubCommand getSubCommand(String group, @NonNull String name) {
        for (BatSubCommand subCommand : subCommands) {
            String commandGroup = subCommand.getGroupName();

            // Check if groups match (both null or both equal)
            boolean groupsMatch = (group == null && commandGroup == null) ||
                    (commandGroup != null && commandGroup.equals(group));
            if (subCommand.getName().equals(name) && groupsMatch) {
                return subCommand;
            }
        }
        return null;
    }
}