package cc.fascinated.bat.command;

import cc.fascinated.bat.common.EnumUtils;
import cc.fascinated.bat.common.InteractionResponder;
import cc.fascinated.bat.common.model.BatGuild;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.events.interaction.command.CommandAutoCompleteInteractionEvent;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.Command;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * A base Rufus command.
 *
 * @author Braydon
 */
public interface IBatCommand {
    /**
     * Get the name of this command.
     *
     * @return the command name
     */
    @NonNull String getName();

    /**
     * Get the full name of this command.
     *
     * @return the full name
     */
    @NonNull String getFullName();

    /**
     * Get the description of this command.
     *
     * @return the command description
     */
    @NonNull String getDescription();

    /**
     * Get the mentionable name of this command.
     *
     * @return the mentionable name
     */
    @NonNull String getMentionable();

    /**
     * Fired when this command is executed.
     * <p>
     * If this command has {@link BatSubCommand}'s, this
     * will not be invoked for the parent, and instead only
     * for the sub-commands.
     * </p>
     *
     * @param guild        the ticket guild that executed this command
     * @param discordGuild the Discord guild that executed this command
     * @param member       the member that executed this command
     * @param responder    the responder to the interaction
     * @param event        the event that triggered this command
     */
    default void execute(@NonNull BatGuild guild, @NonNull Guild discordGuild, @NonNull Member member, @NonNull InteractionResponder responder, @NonNull SlashCommandInteractionEvent event) {
        event.deferReply().queue(); // Thinking by default
    }

    /**
     * Auto complete this command.
     *
     * @param guild the ticket guild that executed this command
     * @param discordGuild the Discord guild that executed this command
     * @param member the member that executed this command
     * @param event the event that triggered this auto completion
     * @return the choices to auto complete
     */
    @NonNull
    default List<Command.Choice> autoComplete(@NonNull BatGuild guild, @NonNull Guild discordGuild, @NonNull Member member, @NonNull CommandAutoCompleteInteractionEvent event) {
        return List.of();
    }

    /**
     * Get the choices for an enum.
     *
     * @param enumClass the enum class to get the choices for
     * @return the enum choices
     * @param <T> the enum type
     */
    @NonNull
    default <T extends Enum<T>> List<Command.Choice> getEnumChoices(@NonNull Class<T> enumClass) {
        return Arrays.stream(enumClass.getEnumConstants())
                .map(constant -> new Command.Choice(EnumUtils.getEnumName(constant), constant.name()))
                .collect(Collectors.toList());
    }
}