package cc.fascinated.bat.command.command;

import cc.fascinated.bat.common.BatEmoji;
import cc.fascinated.bat.common.Colors;
import cc.fascinated.bat.common.InteractionResponder;
import cc.fascinated.bat.common.MiscUtils;
import cc.fascinated.bat.common.model.BatGuild;
import cc.fascinated.bat.service.CommandService;
import cc.fascinated.bat.service.DiscordService;
import jakarta.annotation.Nonnull;
import lombok.NonNull;
import cc.fascinated.bat.command.CommandCategory;
import cc.fascinated.bat.command.IBatCommand;
import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.BatSubCommand;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.emoji.Emoji;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.events.interaction.component.ButtonInteractionEvent;
import net.dv8tion.jda.api.events.interaction.component.StringSelectInteractionEvent;
import net.dv8tion.jda.api.interactions.components.ActionRow;
import net.dv8tion.jda.api.interactions.components.LayoutComponent;
import net.dv8tion.jda.api.interactions.components.buttons.Button;
import net.dv8tion.jda.api.interactions.components.selections.StringSelectMenu;
import org.jetbrains.annotations.NotNull;
import org.springframework.boot.info.BuildProperties;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author Braydon
 */
public final class HelpCommand extends BatCommand {
    private static final String CATEGORY_DROPDOWN_ID = "select-category";
    private static final String MAIN_MENU_BUTTON_ID = "main-help-menu";

    /**
     * The command service to use for the registry.
     */
    @NonNull private final CommandService commandService;

    /**
     * The build properties for this app, null if not available.
     */
    private final BuildProperties buildProperties;

    public HelpCommand(@NonNull CommandService commandService, BuildProperties buildProperties) {
        super("helpbob", "View the commands for Rufus", CommandCategory.GENERAL);
        this.commandService = commandService;
        this.buildProperties = buildProperties;
    }

    /**
     * Fired when this command is executed.
     * <p>
     * If this command has {@link BatSubCommand}'s, this
     * will not be invoked for the parent, and instead only
     * for the sub-commands.
     * </p>
     *
     * @param guild        the ticket guild that executed this command
     * @param discordGuild the Discord guild that executed this command
     * @param member       the member that executed this command
     * @param responder    the responder to the interaction
     * @param event        the event that triggered this command
     */
    @Override
    public void execute(@NonNull BatGuild guild, @NonNull Guild discordGuild, @NonNull Member member, @NonNull InteractionResponder responder, @NonNull SlashCommandInteractionEvent event) {
        responder.reply(true, buildMainMenu(member, discordGuild), buildMainMenuComponents(discordGuild).toArray(new LayoutComponent[0])).queue();
    }

    @Override
    public void onStringSelectInteraction(@NotNull StringSelectInteractionEvent event) {
        // Switch to the selected category
        if (event.getComponentId().equals(CATEGORY_DROPDOWN_ID)) {
            event.editMessageEmbeds(buildCategoryEmbed(Objects.requireNonNull(event.getMember()), CommandCategory.valueOf(event.getSelectedOptions().getFirst().getValue())))
                    .setComponents(
                            ActionRow.of(buildCategoryMenu()),
                            ActionRow.of(Button.primary(MAIN_MENU_BUTTON_ID, "Main Menu").withEmoji(Emoji.fromUnicode("🏠")))
                    ).queue();
        }
    }

    @Override
    public void onButtonInteraction(@NotNull ButtonInteractionEvent event) {
        // Switch to the main menu
        if (event.getComponentId().equals(MAIN_MENU_BUTTON_ID)) {
            event.editMessageEmbeds(buildMainMenu(Objects.requireNonNull(event.getMember()),
                    Objects.requireNonNull(event.getGuild()))).setComponents(buildMainMenuComponents(event.getGuild())).queue();
        }
    }

    /**
     * Build the embed for the main menu.
     *
     * @param member the member to build for
     * @param discordGuild the discord guild
     * @return the main menu
     */
    @NonNull
    private MessageEmbed buildMainMenu(@NonNull Member member, @NonNull Guild discordGuild) {
        String categoryNames = Arrays.stream(CommandCategory.values())
                .map(category -> "- **%s Commands**: `%s`".formatted(MiscUtils.capitalize(category.name()), commandService.getCommands(member, category).size()))
                .collect(Collectors.joining("\n"));

        // Send the help menu
        return new EmbedBuilder()
                .setColor(Colors.DEFAULT)
                .setTitle(BatEmoji.RUFUS + " Rufus Help Menu")
                .setDescription(MiscUtils.arrayToString(
                        "Rufus is a smart and efficient Discord bot that streamlines ticket management for your server. With a focus on simplicity and responsiveness, Rufus helps your staff team handle support requests, bug reports, and general inquiries with ease.",
                        "",
                        "Here are my commands:",
                        categoryNames,
                        "",
                        "To get a list of commands for an individual category, select it from the dropdown below. Interested and wanna use me in your server? [Invite Me](" + MiscUtils.getInviteUrl(discordGuild.getJDA()) + ")!"
                )).setFooter("Rufus v%s".formatted(buildProperties == null ? "0" : buildProperties.getVersion())).build();
    }

    /**
     * Build the components for the main menu.
     *
     * @param guild the guild to use for the JDA instance
     * @return the components
     */
    @NonNull
    private List<LayoutComponent> buildMainMenuComponents(@NonNull Guild guild) {
        return Arrays.asList(
                ActionRow.of(buildCategoryMenu()),
                ActionRow.of(
                        Button.link("https://rufus.rainnny.club", "Support Server").withEmoji(Emoji.fromUnicode("🏠")),
                        Button.link(MiscUtils.getInviteUrl(guild.getJDA()), "Invite Me").withEmoji(Emoji.fromUnicode("🤖"))
                )
        );
    }

    /**
     * Build the dropdown menu for categories.
     *
     * @return the category dropdown menu
     */
    @NonNull
    private StringSelectMenu buildCategoryMenu() {
        StringSelectMenu.Builder categoryDropdown = StringSelectMenu.create(CATEGORY_DROPDOWN_ID).setPlaceholder("👉🏼 Select a category...");
        for (CommandCategory category : CommandCategory.values()) {
            categoryDropdown.addOption(MiscUtils.capitalize(category.name()) + " Commands", category.name(), category.getEmoji());
        }
        return categoryDropdown.build();
    }

    /**
     * Build the embed for the category menu.
     *
     * @param member   the member to build for
     * @param category the category to build for
     * @return the category menu
     */
    @NonNull
    private MessageEmbed buildCategoryEmbed(@NonNull Member member, @NonNull CommandCategory category) {
        // Build the command list
        StringBuilder commands = new StringBuilder();
        for (IBatCommand command : commandService.getCommands(member, category)) {
            // Skip parents that have sub-commands
            if (command instanceof BatCommand parentCommand && !parentCommand.getSubCommands().isEmpty()) {
                continue;
            }
            commands.append("- ").append(command.getMentionable()).append(" - ").append(command.getDescription()).append("\n");
        }

        // Build the embed
        return new EmbedBuilder()
                .setColor(Colors.DEFAULT)
                .setTitle(category.getEmoji().getFormatted() + " " + MiscUtils.capitalize(category.name()) + " Commands")
                .setDescription(MiscUtils.arrayToString(
                        "Here are the commands for the " + MiscUtils.capitalize(category.name()) + " category:",
                        "",
                        commands.toString()
                )).setFooter("Rufus v%s".formatted(buildProperties == null ? "0" : buildProperties.getVersion())).build();
    }
}